<?php
/**
 * This Software is the property of Data Development and is protected
 * by copyright law - it is NOT Freeware.
 *
 * Any unauthorized use of this software without a valid license
 * is a violation of the license agreement and will be prosecuted by
 * civil and criminal law.
 *
 * http://www.shopmodule.com
 *
 * @copyright (C) D3 Data Development (Inh. Thomas Dartsch)
 * @author    D3 Data Development <support@shopmodule.com>
 * @link      http://www.oxidmodule.com
 */

class d3_multivariants_update extends d3install_updatebase
{
    public $sModKey = 'd3_multivariants';
    public $sModName = 'mehrdimensionale Varianten';
    public $sModVersion = '3.0.2.1';
    public $sModRevision = '148';
    public $sBaseConf = '5nwS2Y0c1RQRHRIUG9MYjZzQkJIdkFZMHpDcG9rVlZPUWxNZHIxYWlhenZnWXo4Y1J2WUxRZk0xN2Eze
UlBVHFRN1hmcWRuUG00elZmSGMydVQ1WkQ2amlaMmwvcXI5Ulc4TmZ0N0pQYnh2K1ZjaWl4V3V1aTFkN
3dFNURWVm1sVVFMQlBYTFlkWmVBYytFazY1WTRDbmpCZFBNa0RhTVBNMmpjYll3VUFnRTNOOTRZUDQzN
Fd3OEMzcFlxaWN5K1hsZndvalpLWDh0bTFIb1Z1Z1BnRDJ4U3VNRzlCcGFMTFNWNTI4bFR5UjVTTWRpV
EE4TS9INm8rMklnY0F3K1BvaUJiTmsxUzNxSHlXcGFMYXB5N2RkUzdDS2lWSEcvVUUwVG1PMkZnMDErc
1k9';
    public $sRequirements = '<?xml version="1.0" encoding="UTF-8"?>
      <jobs>
        <registerModule>
          <module class="article_variant" modulepath="d3/d3_multivariants/modules/controllers/admin/d3_article_variant_mv" />
        </registerModule>
      </jobs>';

    public $sBaseValue = '';

    protected $_aUpdateMethods = array(
        array(  // Moduleintrag anlegen, wenn noch nicht vorhanden
            'check' => 'checkModCfgItemExist',
            'do'    => 'updateModCfgItemExist'
        ),
        array(  // exact one whitespace between separator and variantselection
            'check' => 'checkVariantWhitespaces',
            'do'    => 'updateVariantWhitespaces'
        ),
        array( // alte Moduldateien entfernen
            'check' => 'hasOldModuleFiles',
            'do'    => 'deleteOldModuleFiles'
        ),
        array(
            'check' => 'hasUnregisteredFiles',
            'do'    => 'showUnregisteredFiles'
        ),
        array(  // Moduleintrag aktualisieren
            'check' => 'checkModCfgSameRevision',
            'do'    => 'updateModCfgSameRevision'
        ),
    );

    protected $_aRefreshMetaModuleIds = array('d3_multivariants');

    public $aOldModuleFiles = array(
        'd3/d3_multivariants/d3_mv_stub.php',
        'd3/d3_multivariants/models/d3_multivariants_update.php',
        'd3/d3_multivariants/modules/controllers/admin/d3_cfg_multivariants.php',
        'd3/d3_multivariants/modules/controllers/admin/d3_cfg_multivariants_licence.php',
        'd3/d3_multivariants/modules/controllers/admin/d3_cfg_multivariants_list.php',
        'd3/d3_multivariants/modules/controllers/admin/d3_cfg_multivariants_main.php',
        'd3/d3_multivariants/modules/models/d3_oxarticle_mv.php',
        // alte Verzeichnisse (werden zustzlich auf empty geprft, bevor eine Lschung stattfindet)
        'd3/d3_multivariants/models/',
        'd3/d3_multivariants/modules/models/',
        'd3/d3_multivariants/out/admin/src/js/d3_cfg_multivariants.js',
    );

    /**********************************************************************/
    /*** individuelle Updateprozesse **************************************/
    /**********************************************************************/

    /**
     * @return bool
     */
    public function checkVariantWhitespaces()
    {
        return $this->_checkUpdateFile('d3/d3_multivariants/setup/d3multiVariants_whitespaceQuerys.php');
    }

    /**
     * @return bool
     */
    public function updateVariantWhitespaces()
    {
        // we dont need this configparam
        $blVariantConfigParam = oxRegistry::getConfig()->getConfigParam('d3multivariantsupdate');

        if (false == empty($blVariantConfigParam)) {
            // delete it if available
            $oDb = oxDb::getDb();
            $sShopId = $oDb->quote($this->getShopId());

            $sQ = " DELETE FROM oxconfig WHERE oxshopid = " . $sShopId //
                . " AND oxvarname = 'd3multivariantsupdate' " //
                . " AND oxmodule = 'multiVariants_3.x'";
            $oDb->execute($sQ);
        }

        $blReturn = $this->_doUpdateFile('d3/d3_multivariants/setup/d3multiVariants_whitespaceQuerys.php');

        return $blReturn;
    }

    /**********************************************************************/
    /*** Modul Eintrag in der Datenbank ***********************************/
    /**********************************************************************/
    /**
     * @return bool
     */
    public function checkModCfgItemExist()
    {
        $blRet = false;
        foreach ($this->getShopList() as $oShop) {
            /** @var $oShop oxshop */
            $aWhere = array(
                'oxmodid'       => $this->sModKey,
                'oxnewrevision' => $this->sModRevision,
                'oxshopid'      => $oShop->getId(),
            );

            $blRet = $this->_checkTableItemNotExist('d3_cfg_mod', $aWhere);

            if ($blRet) {
                return $blRet;
            }
        }

        return $blRet;
    }

    /**
     * @return bool
     */
    public function updateModCfgItemExist()
    {
        $blRet = false;

        if ($this->checkModCfgItemExist()) {
            foreach ($this->getShopList() as $oShop) {
                /** @var $oShop oxshop */
                $aWhere = array(
                    'oxmodid'       => $this->sModKey,
                    'oxshopid'      => $oShop->getId(),
                    'oxnewrevision' => $this->sModRevision,
                );

                if ($this->_checkTableItemNotExist('d3_cfg_mod', $aWhere)) {
                    // update don't use this property
                    unset($aWhere['oxnewrevision']);

                    $aInsertFields = array(
                        'OXID'           => array(
                            'content'      => "md5('" . $this->sModKey . " " . $oShop->getId() . "')",
                            'force_update' => false,
                            'use_quote'    => false,
                        ),
                        'OXSHOPID'       => array(
                            'content'      => $oShop->getId(),
                            'force_update' => false,
                            'use_quote'    => true,
                        ),
                        'OXMODID'        => array(
                            'content'      => $this->sModKey,
                            'force_update' => true,
                            'use_quote'    => true,
                        ),
                        'OXNAME'         => array(
                            'content'      => $this->sModName,
                            'force_update' => true,
                            'use_quote'    => true,
                        ),
                        'OXACTIVE'       => array(
                            'content'      => "0",
                            'force_update' => false,
                            'use_quote'    => false,
                        ),
                        'OXBASECONFIG'   => array(
                            'content'      => $this->sBaseConf,
                            'force_update' => true,
                            'use_quote'    => true,
                        ),
                        'OXINSTALLDATE'  => array(
                            'content'      => "NOW()",
                            'force_update' => true,
                            'use_quote'    => false,
                        ),
                        'OXVERSION'      => array(
                            'content'      => $this->sModVersion,
                            'force_update' => true,
                            'use_quote'    => true,
                        ),
                        'OXSHOPVERSION'  => array(
                            'content'      => oxRegistry::getConfig()->getEdition(),
                            'force_update' => true,
                            'use_quote'    => true,
                        ),
                        'OXREQUIREMENTS' => array(
                            'content'      => $this->sRequirements,
                            'force_update' => true,
                            'use_quote'    => true,
                        ),
                        'OXVALUE'        => array(
                            'content'      => $this->sBaseValue,
                            'force_update' => false,
                            'use_quote'    => true,
                        ),
                        'OXNEWREVISION'  => array(
                            'content'      => $this->sModRevision,
                            'force_update' => true,
                            'use_quote'    => true,
                        ),
                    );
                    $aRet          = $this->_updateTableItem2('d3_cfg_mod', $aInsertFields, $aWhere);
                    $blRet         = $aRet['blRet'];
                    $this->setActionLog('SQL', $aRet['sql'], __METHOD__);
                    $this->setUpdateBreak(false);

                    if ($this->getStepByStepMode()) {
                        break;
                    }
                }
            }
        }

        return $blRet;
    }

    /**
     * @return bool
     */
    public function checkModCfgSameRevision()
    {
        return $this->_checkModCfgSameRevision($this->sModKey);
    }

    /**
     * @return bool
     */
    public function updateModCfgSameRevision()
    {
        $blRet = false;

        if ($this->checkModCfgSameRevision()) {
            $aRet = $this->_updateModCfgSameRevision($this->sModKey);

            $this->setActionLog('SQL', $aRet['sql'], __METHOD__);
            $this->setUpdateBreak(false);
            $blRet = $aRet['blRet'];
        }

        return $blRet;
    }

    /**
     * @return bool
     */
    public function hasUnregisteredFiles()
    {
        return $this->_hasUnregisteredFiles($this->sModKey, array('d3FileRegister'));
    }

    /**
     * @return bool
     */
    public function showUnregisteredFiles()
    {
        return $this->_showUnregisteredFiles($this->sModKey, array('d3FileRegister'));
    }
}